@@ -1,4 +1,4 @@
 .onAttach <- function(libname, pkgname) {
-  packageStartupMessage("Please take a look at the vignette for tips/suggestions/examples of using the breakfast package.")
+  packageStartupMessage("Please see the vignette for tips/suggestions/examples of using the breakfast package.")
   #  message("Please take a look at the vignette for tips/suggestions/examples of using the breakfast package.")
 }
\ No newline at end of file
